local addonName, addonTable = ...

-- Adds to the addonTable:
--  DrawTooltip(enabled)
--  UpdateTooltip()
--  IsTooltipShown()
-- Uses from the addonTable:
--  GetMobLevel()
--  IsDualWielding()
--  hitTable

--------------------------
-- Constants -------------
--------------------------
local OUTCOMES = {
	"MISS",
	"DODGE",
	"PARRY",
	"BLOCK",
	"CRIT",
	"CRUSHING",
	"HIT",
}

local COLORS = {
	["MISS"] = { r=0.41, g=0.72, b=0.31 },
	["DODGE"] = { r=0.41, g=0.72, b=0.31 },
	["PARRY"] = { r=0.41, g=0.72, b=0.31 },
	["BLOCK"] = { r=0, g=1, b=0 },
	["CRIT"] = { r=1, g=0, b=0 },
	["CRUSHING"] = { r=1, g=0.5, b=0 },
	["HIT"] = { r=1, g=1, b=0 }
}
local titleColor = { r=1, g=0.82, b=0 }
local headerColor = { r=1, g=1, b=0.6 }

--------------------------
-- Variables -------------
--------------------------
local Addon = LibStub("AceAddon-3.0"):GetAddon(addonName)
local L = LibStub("AceLocale-3.0"):GetLocale(addonName, false)
local LibQTip = LibStub("LibQTip-1.0")
local LDBObject = LibStub("LibDataBroker-1.1"):GetDataObjectByName(addonName)
local tooltip

--------------------------
-- Upvalues --------------
--------------------------
local GetMobLevel, IsDualWielding, hitTable = addonTable.GetMobLevel, addonTable.IsDualWielding, addonTable.hitTable
local format = string.format

--------------------------
-- Functions -------------
--------------------------
local function UpdateTooltip()
	local line = 4
	local color
	local empty = 0
	for _, v in pairs(OUTCOMES) do
		color = COLORS[v]
		if hitTable[v] > 0.0 then
			tooltip:SetCell(line, 1, L[v], color)
			tooltip:SetCell(line, 2, format("%.2f", hitTable[v]), color)
			line = line + 1
		else
			empty = empty + 1
		end
	end
	for i=1,empty do
		tooltip:SetCell(line, 1, "")
		tooltip:SetCell(line, 2, "")
		line = line + 1
	end

	line = line + 2
	if(IsDualWielding()) then
		tooltip:SetCell(line, 1, L["DWTEXT"], 2, headerColor)
	else
		tooltip:SetCell(line, 1, "", 2, headerColor)
	end
end
addonTable.UpdateTooltip = UpdateTooltip

local function DrawTooltip(enabled)
	local line
	tooltip:Clear()
	tooltip:SetCellMarginV(0)
	tooltip:AddHeader()
	tooltip:SetCell(1, 1, L["TITLE"], "CENTER", 2, titleColor)
	tooltip:AddSeparator(5, 0, 0, 0, 0)
	if enabled then
		tooltip:AddLine()
		tooltip:SetCell(3, 1, L["OUTCOME"], headerColor)
		tooltip:SetCell(3, 2, L["CHANCE"], headerColor)
		for i=1,#OUTCOMES do
			tooltip:AddLine()
		end
		tooltip:AddSeparator(5, 0, 0, 0, 0)
		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, format(L["LEVELTEXT"], GetMobLevel()), 2, headerColor)
		line = tooltip:AddLine()
		tooltip:AddSeparator(5, 0, 0, 0, 0)
		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["ENABLED_NOTE"], GameTooltipTextSmall, 2, titleColor)
		UpdateTooltip()
	else
		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["DISABLED_NOTE"], GameTooltipTextSmall, 2, titleColor)
	end
end
addonTable.DrawTooltip = DrawTooltip

function addonTable.IsTooltipShown()
	return tooltip ~= nil
end

-- Custom Cellprovider - adds vertical padding and coloring
local myProvider, cellPrototype, baseCellPrototype = LibQTip:CreateCellProvider(LibQTip.LabelProvider)

function cellPrototype:SetupCell(tooltip, value, justification, font, color, ...)
	-- Call the base method
	local width, height = baseCellPrototype.SetupCell(self, tooltip, value, justification, font)
  
	-- The base LabelProvider stores its FontString in the fontString attribute
	local fs = self.fontString
  
	-- Set color
	if color and type(color) == 'table' then
		fs:SetTextColor(color.r, color.g, color.b)
	end
  
	-- Unless this is the first line or an empty line, add padding of 3
	if self:GetPosition() > 1 then
		if height > 1.5 then -- For some reason, empty cells have a height of  like 1.0028235
			fs:SetPoint("TOPLEFT", self, "TOPLEFT", 0, -3)
			height = height + 3
		else
			height = 0
		end
	end
  
	-- Return the dimensions
	return width, height
end

function cellPrototype:ReleaseCell()
	-- Revert to default color
	self.fontString:SetTextColor(1, 1, 1)
end

function LDBObject.OnEnter(button)
	tooltip = LibQTip:Acquire(addonName, 2, "LEFT", "RIGHT")
	tooltip:SetDefaultProvider(myProvider)
	DrawTooltip(Addon:IsEnabled())
	tooltip:SmartAnchorTo(button)
	tooltip:Show()
end

function LDBObject.OnLeave()
	LibQTip:Release(tooltip)
	tooltip = nil
end
